package classifier;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.StaxDriver;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;
import interfaces.IDataset;
import helper.Parser;
 
// ##kdyz radek zacina dvojkrizem, bude to komentar
// #! pocet vstupu, pocet vystupu 3 4 
// a hodnoty
// Vezmeme si objekt ktery umi cist po radcich a nasledne radek rozparsujeme
// Hledam dvojkriz vykrickin, prectu hodnoty, alokuju pole
// Potom prectu radek, kterej bude hodnotama, dam do Stringu, Splitnu, konvertuju do Double a naplnim pole. 
//HW
// vypis vstupu a vystupu
// ulozeni datasetu do XML, csv a txt; 
/**
 *
 * @author User
 */
public class Dataset implements IDataset {
 
    private boolean samplesBeingReturned = false;
 
    private int access;
    private int lastReturnedSampleIndex;
 
    private ArrayList<Sample> samples;
    private ArrayList<Sample> nonReturnedSamples;
 
    private Random random;
 
    public Dataset() {
        this.samples = new ArrayList<Sample>();
        this.lastReturnedSampleIndex = 0;
        this.nonReturnedSamples = new ArrayList<Sample>();
    }
 
    public void addSample(Sample sample) {
        this.samples.add(sample);
    }
 
    @Override
    public void clearSamples() {
        this.samples.clear();
    }
 
    public ArrayList<Sample> getSamples() {
        return this.samples;
    }
 
    public void print(PrintStream out) {
        for (int i = 0; i < samples.size(); i++) {
            samples.get(i).print(out);
        }
    }
 
    //# komentář - dvojkříž na prvním znaku řádku
    //#! 3 4           metainformace, počet vstup; a počet výstupů (platí pro celý dataset)
    //3.1 2.4 5.1 7.7 0.2 0.1 1.0         hodnoty (7 hodnot - 3 + 4)
    //4.5 1.2 0.2 0.2 0.2 0.1 0.0
    //... atd.
    //
    //Použije se pouze desetinná tečka, převod string->double by neměl být závislý na nastaveném Locale.
    //    
    @Override
    public void loadFromText(String filename) {
        int inputCount = 0;
        int outputCount = 0;
 
        double[] inputs;
        double[] outputs;
 
        try (BufferedReader br = new BufferedReader(new FileReader(filename))) {
            String line;
            while ((line = br.readLine()) != null) {
                switch (line.charAt(0)) {
                    // the has 
                    case '#':
                        switch (line.charAt(1)) {
                            case '#':
                                // comment
                                break;
                            case '!':
                                String[] IOCount = line.split("\\s+");
                                for (int i = 0; i < IOCount.length - 1; i++) {
                                    if (Parser.tryParseInt(IOCount[i]) && Parser.tryParseInt(IOCount[i + 1])) {
                                        inputCount = Integer.parseInt(IOCount[i]);
                                        outputCount = Integer.parseInt(IOCount[i + 1]);
                                    }
                                }
                                break;
                            default:
                                // something unuseful or values?
                                break;
                        }
                        break;
                    // values probably
                    default:
                        if (inputCount != 0 && outputCount != 0) {
 
                            inputs = new double[inputCount];
                            outputs = new double[outputCount];
 
                            String[] values = line.split("\\s+");
                            for (int i = 0; i < inputCount + outputCount; i++) {
                                if (i < inputCount) {
                                    inputs[i] = Double.parseDouble(values[i]);
                                } else {
                                    outputs[i - inputCount] = Double.parseDouble(values[i]);
                                }
                            }
                            Sample newSample = new Sample(inputs, outputs);
                            this.samples.add(newSample);
                        }
                        break;
                }
            }
        } catch (IOException ex) {
            Logger.getLogger(Dataset.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
 
    @Override
    public void loadFromCsv(String filename) {
        throw new UnsupportedOperationException("Not supported yet.");
    }
 
    @Override
    public void load(String filename) {
        try {
            FileInputStream fis = new FileInputStream(filename);
            XStream xstream = new XStream(new StaxDriver());
 
            samples = (ArrayList<Sample>) xstream.fromXML(fis);
 
        } catch (FileNotFoundException ex) {
            Logger.getLogger(Dataset.class
                    .getName()).log(Level.SEVERE, null, ex);
        }
    }
 
    @Override
    public void save(String filename) {
 
        try {
            FileOutputStream fos = new FileOutputStream(filename);
            XStream xstream = new XStream(new StaxDriver());
 
            xstream.toXML(samples, fos);
 
        } catch (FileNotFoundException ex) {
            Logger.getLogger(Dataset.class
                    .getName()).log(Level.SEVERE, null, ex);
        }
 
    }
 
    @Override
    public void saveToTextFile(String filename) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }
 
    @Override
    public void saveToCsvFile(String filename) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }
 
    @Override
    public int getSize() {
        return samples.size();
    }
 
    @Override
    public void randomInit(long seed, int access) {
        random = new Random(seed);
        this.access = access;
    }
 
    @Override
    public Sample getRandom() {
        Sample sampleToReturn;
        int randomIndex;
 
        switch (this.access) {
            case 0: // sekvenční přístup v pořadí vzorů v datasetu
                sampleToReturn = samples.get(this.lastReturnedSampleIndex);
                if (this.lastReturnedSampleIndex < samples.size() - 1) {
                    this.lastReturnedSampleIndex += 1;
                }
 
                break;
            default: // case 1: náhodný přístup s opakováním (vygenerujte náhodný index v rozsahu počtu vzorů v datasetu a vzor na tomto indexu vraťte)
                randomIndex = random.nextInt(samples.size());
                sampleToReturn = samples.get(randomIndex);
                break;
            case 2: // náhodný přístup bez opakování (veďte si seznam vzorů, které ještě nebyly vydány a v nich náhodně generujte index)
                if (nonReturnedSamples.isEmpty()) {
                    if (samplesBeingReturned == false) {
                        nonReturnedSamples = (ArrayList<Sample>) samples.clone();
                        samplesBeingReturned = true;
                    } else {
                        // Asign again or return null?
                        // nothing to return
                         
                        // Will be RESET
                        return null;
                    }
                }
                randomIndex = random.nextInt(nonReturnedSamples.size());
                sampleToReturn = nonReturnedSamples.get(randomIndex);
                nonReturnedSamples.remove(sampleToReturn);
                break;
        }
        return sampleToReturn;
    }
 
    @Override
    public void append(Dataset dataset) {
        for (Sample sample : dataset.samples) {
            this.samples.add(sample);
        }
    }
 
    @Override
    public void removeSample(int index) {
        samples.remove(index);
    }
 
    @Override
    public void removeSample(Sample sample) {
        samples.remove(sample);
    }

}