package neuron.perceptron;

import classifier.Dataset;
import classifier.Sample;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.util.Random;
import neuron.InputLayer;

import neuron.Layer;
import neuron.Network;
import neuron.Neuron;

public class PerceptronNetwork extends Network {

    private PrintStream out;

    public PerceptronNetwork() throws FileNotFoundException {
        out = new PrintStream(new File("C:\\Users\\Lukas\\Desktop\\SVITest.txt"));
    }

    public double Learn(Dataset data, double weightMin, double weightMax, Random rand, double eta, double alhpa, double minError, int nIterations) throws Exception {
        //TESTING -------------------------

        initWeights(weightMin, weightMax, rand);

        double err = 1;

        for (int i = 1; i < layers.size(); i++) {
            ((PerceptronLayer) layers.get(i)).initLearning();
        }

        out.printf("# inicializace vah \n");
        printForTesting(out);

        for (int i = 0; i < nIterations; i++) {
            out.printf("# epocha " + (i+1) + "\n");

            err = learnEpoch(data, eta, alhpa);

            System.out.println(err);

            if (err <= minError) {
                break;
            }
        }

        System.out.println(err);
        return err;
    }

    public void Learn() {
        for (int i = 1; i < layers.size(); i++) {
            ((PerceptronLayer) layers.get(i)).learn();
        }
    }

    double learnEpoch(Dataset data, double eta, double alpha) throws FileNotFoundException, Exception {
        //TESTING -------------------------
//        out = new PrintStream(new File("C:\\Users\\Lukas\\Desktop\\SVITest.txt"));
        int index = 1;

        double error = 0;

        for (int i = 1; i < layers.size(); i++) {
            ((PerceptronLayer) layers.get(i)).initEpoch();
        }

        for (Sample sample : data.getSamples()) {
            out.printf("# iterace " + (index) + "\n");
            index++;

            ((InputLayer) layers.get(0)).setInputs(sample.getInputs());

            out.printf("%20.15f ; input [0]\n", sample.inputs[0]);
            out.printf("%20.15f ; input [1]\n", sample.inputs[1]);

//
//               0,000000000000000 ; input [0]
//   0,000000000000000 ; input [1]
//            
            // Propagace do site
            recall();

            // Vypocet chyby z vystupni vrstvy
            // Nem?l jsem +, zjistit
            error += ((PerceptronLayer) layers.get(layers.size() - 1)).outputDelta(sample.getOutputs());

            // Zpetna propagace
            backPropagate(); //Predpoklad: delty jsou ve vystupni vrstve

            //Uceni
            Learn(); // Predpoklad: delty jsou uz ve vsech vrstvach
            printForTesting(out);
        }

        for (int i = 1; i < layers.size(); i++) {
            ((PerceptronLayer) layers.get(i)).endEpoch(eta, alpha);
        }

        return error / data.getSize();
    }

    public void backPropagate() {
        for (int i = layers.size() - 1; i > 1; i--) {
            ((PerceptronLayer) layers.get(i)).backPropagate();
        }
    }

    @Override
    public void printActualState(PrintStream out) {
        for (Layer layer : getLayers()) {
            for (Neuron neuron : layer.getNeurons()) {
                PerceptronNeuron perceptron = null;
                if (neuron instanceof PerceptronNeuron) {
                    perceptron = (PerceptronNeuron) neuron;
                }
                if (perceptron != null) {
                    for (double weight : perceptron.getWeights()) {
                        out.printf("%7.5f", weight);
                    }
                    out.printf("%7.5f", perceptron.getTreshold());
                    out.print("\t \t");
                }
            }
        }
    }

    public void printForTesting(PrintStream out) throws Exception {
        if (layers.size() != 3) {
            throw new Exception("incompatible network for testing");
        }
        PerceptronLayer output = (PerceptronLayer) layers.get(2);
        PerceptronLayer hidden = (PerceptronLayer) layers.get(1);
        InputLayer input = (InputLayer) layers.get(0);
        if (input.neurons.length != 2
                || hidden.neurons.length != 2
                || output.neurons.length != 1) {
            throw new Exception("incompatible network for testing");
        }
        PerceptronNeuron pn = (PerceptronNeuron) output.neurons[0];
        out.printf("%20.15f ; output: y \n", pn.y);
        out.printf("%20.15f ; output: threshold\n", pn.treshold);
        for (int j = 0; j < 2; j++) {
            out.printf("%20.15f ; output: w[%d]\n", pn.weights[j], j);
        }
        out.printf("%20.15f ; output: delta\n", pn.delta);
        out.printf("%20.15f ; output: delta threshold\n", pn.deltaTheta);
        for (int j = 0; j < 2; j++) {
            out.printf("%20.15f ; output: delta w[%d]\n", pn.deltaW[j], j);
        }
        for (int j = 0; j < 2; j++) {
            out.printf("%20.15f ; hidden: y[%d]\n",
                    hidden.neurons[j].y, j);
        }
        for (int j = 0; j < 2; j++) {
            out.printf("%20.15f ; hidden: threshold [%d]\n",
                    ((PerceptronNeuron) hidden.neurons[j]).treshold, j);
        }
        for (int i = 0; i < 2; i++) {
            for (int j = 0; j < 2; j++) {
                out.printf("%20.15f ; hidden: w[%d][%d]\n",
                        ((PerceptronNeuron) hidden.neurons[i]).weights[j], i, j);
            }
        }
        for (int j = 0; j < 2; j++) {
            out.printf("%20.15f ; hidden: delta [%d]\n",
                    ((PerceptronNeuron) hidden.neurons[j]).delta, j);
        }
        for (int j = 0; j < 2; j++) {
            out.printf("%20.15f ; hidden: delta threshold [%d]\n",
                    ((PerceptronNeuron) hidden.neurons[j]).deltaTheta, j);
        }
        for (int i = 0; i < 2; i++) {
            for (int j = 0; j < 2; j++) {
                out.printf("%20.15f ; hidden: delta w[%d][%d]\n",
                        ((PerceptronNeuron) hidden.neurons[i]).deltaW[j], i, j);
            }
        }
        for (int j = 0; j < 2; j++) {
            out.printf("%20.15f ; hidden: old delta threshold [%d]\n",
                    ((PerceptronNeuron) hidden.neurons[j]).oldDeltaTheta, j);
        }
        for (int i = 0; i < 2; i++) {
            for (int j = 0; j < 2; j++) {
                out.printf("%20.15f ; hidden: old delta w[%d][%d]\n",
                        ((PerceptronNeuron) hidden.neurons[i]).oldDeltaW[j], i, j);
            }
        }
    }
    // Na rovni st, nejdv inicializace uen, inicializace vah, inicializace epochy, pak bude loop. 
    // Respektive bude smyka i ped inicializac epochy, uvnit bude smyuka pes vechny vzoru trnovac mnoiny,
    // kde bude recall, potom bude vpoet delta na vstupn vrstv, pot zavolme learning na vech vrstvch, kde bude backPropage. 

    //Set inputs a nsledn? recall
    
    public void recall(double[] d) {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }
}
